local _, Addon = ...
local L = Addon:GetLocale()

-- Strings for the binding XML
BINDING_HEADER_MOUSELOOKTOGGLE = L["BINDING_HEADER_MOUSELOOKTOGGLE"]
BINDING_NAME_MOUSELOOKTOGGLE = L["BINDING_NAME_MOUSELOOKTOGGLE"]
BINDING_DESC_MOUSELOOKTOGGLE = L["BINDING_NAME_MOUSELOOKTOGGLE"]
BINDING_CATEGORY_MOUSELOOK = L["ADDON_NAME"]


-- Screen settings so we can do placement of the cursor correctly.
local screenWidth, screenHeight = WorldFrame:GetSize()
local midOffsetX = (screenWidth * WorldFrame:GetEffectiveScale()) / 2
local midOffsetY = (screenHeight * WorldFrame:GetEffectiveScale()) / 2

-- Cursor replacement for whenever we are mouselooking so we know where our cursor is, and so we know that we are mouselooking.
local cursorFrame = CreateFrame("Frame", nil, WorldFrame)
cursorFrame:SetFrameStrata("Tooltip")
cursorFrame:SetFrameLevel(0)
cursorFrame:SetWidth(16)
cursorFrame:SetHeight(16)
cursorFrame:SetAlpha(1)
cursorFrame:SetPoint("CENTER", 0, 0);
cursorFrame:SetScript("OnShow", function() Addon:OnCursorShow() end)
local cursorTexture = cursorFrame:CreateTexture("Texture", "Background")
cursorTexture:SetTexture("Interface\\CURSOR\\Crosshairs.blp")
cursorTexture:SetWidth(16)
cursorTexture:SetHeight(16)
cursorTexture:SetAllPoints(cursorFrame)
cursorFrame:Hide()


local starttime = 0

-- Toggle on Mouselook and set our cursor placeholder.  
function Addon:ToggleMouseLook()
    if IsMouselooking() then
        cursorFrame:Hide()
        MouselookStop()
    else
        local x, y = GetCursorPosition()
        MouselookStart()
        starttime = time()
        cursorFrame:SetPoint("CENTER", WorldFrame, x - midOffsetX, y - midOffsetY)
        cursorFrame:Show()
    end
end

-- Expose this to the public namespace
Addon.Public.ToggleMouseLook = Addon.ToggleMouseLook

function Addon:OnCursorUpdate()
    self:HideCursorIfNotMouselooking()
end

-- When we show the cursor, start polling to see when we need to stop showing it.
-- Mouselook mode can be cancelled by the user right-clicking, which is why we need this.
-- There is no event for exiting mouselook, so we must poll for it.
function Addon:OnCursorShow()
    self:CursorTickerStart()
end

function Addon:HideCursorIfNotMouselooking()
    if not IsMouselooking() then
        cursorFrame:Hide()
        self:CursorTickerStop()
    end
end

local cursorTicker = {}
function Addon:CursorTickerStart()
    cursorTicker = C_Timer.NewTicker(.5, function() Addon:HideCursorIfNotMouselooking() end)
end

function Addon:CursorTickerStop()
    if cursorTicker and cursorTicker.Cancel then
        cursorTicker:Cancel()
    end
end


